@@ -55,12 +55,6 @@ Call = namedtuple("Call", ["request", "response"])
 
 _real_send = HTTPAdapter.send
 
-_wrapper_template = """\
-def wrapper%(signature)s:
-    with responses:
-        return func%(funcargs)s
-"""
-
 logger = logging.getLogger("responses")
 
 
@@ -126,30 +120,59 @@ def _cookies_from_headers(headers):
     return cookiejar_from_dict(cookies_dict)
 
 
-def get_wrapped(func, wrapper_template, evaldict):
-    # Preserve the argspec for the wrapped function so that testing
-    # tools such as pytest can continue to use their fixture injection.
+_wrapper_template = """\
+def wrapper%(wrapper_args)s:
+    with responses:
+        return func%(func_args)s
+"""
+
+
+def get_wrapped(func, responses):
     if six.PY2:
         args, a, kw, defaults = inspect.getargspec(func)
+        wrapper_args = inspect.formatargspec(args, a, kw, defaults)
+
+        # Preserve the argspec for the wrapped function so that testing
+        # tools such as pytest can continue to use their fixture injection.
+        if hasattr(func, "__self__"):
+            args = args[1:]  # Omit 'self'
+        func_args = inspect.formatargspec(args, a, kw, None)
     else:
-        args, a, kw, defaults, kwonlyargs, kwonlydefaults, annotations = inspect.getfullargspec(
-            func
-        )
+        signature = inspect.signature(func)
+        signature = signature.replace(return_annotation=inspect.Signature.empty)
+        # If the function is wrapped, switch to *args, **kwargs for the parameters
+        # as we can't rely on the signature to give us the arguments the function will
+        # be called with. For example unittest.mock.patch uses required args that are
+        # not actually passed to the function when invoked.
+        if hasattr(func, "__wrapped__"):
+            wrapper_params = [
+                inspect.Parameter("args", inspect.Parameter.VAR_POSITIONAL),
+                inspect.Parameter("kwargs", inspect.Parameter.VAR_KEYWORD),
+            ]
+        else:
+            wrapper_params = [
+                param.replace(annotation=inspect.Parameter.empty)
+                for param in signature.parameters.values()
+            ]
+        signature = signature.replace(parameters=wrapper_params)
 
-    signature = inspect.formatargspec(args, a, kw, defaults)
-    is_bound_method = hasattr(func, "__self__")
-    if is_bound_method:
-        args = args[1:]  # Omit 'self'
-    callargs = inspect.formatargspec(args, a, kw, None)
-
-    ctx = {"signature": signature, "funcargs": callargs}
-    six.exec_(wrapper_template % ctx, evaldict)
+        wrapper_args = str(signature)
+        params_without_defaults = [
+            param.replace(
+                annotation=inspect.Parameter.empty, default=inspect.Parameter.empty
+            )
+            for param in signature.parameters.values()
+        ]
+        signature = signature.replace(parameters=params_without_defaults)
+        func_args = str(signature)
 
+    evaldict = {"func": func, "responses": responses}
+    six.exec_(
+        _wrapper_template % {"wrapper_args": wrapper_args, "func_args": func_args},
+        evaldict,
+    )
     wrapper = evaldict["wrapper"]
-
     update_wrapper(wrapper, func)
-    if is_bound_method:
-        wrapper = wrapper.__get__(func.__self__, type(func.__self__))
     return wrapper
 
 
@@ -538,8 +561,7 @@ class RequestsMock(object):
         return success
 
     def activate(self, func):
-        evaldict = {"responses": self, "func": func}
-        return get_wrapped(func, _wrapper_template, evaldict)
+        return get_wrapped(func, self)
 
     def _find_match(self, request):
         found = None
